@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: FWObjectDatabase.cpp,v 1.3 2004/09/09 05:35:58 vkurland Exp $
+  $Id: FWObjectDatabase.cpp,v 1.7 2005/02/02 06:32:26 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -395,6 +395,8 @@ Firewall* FWObjectDatabase::findFirewallByName(const string &name) throw(FWExcep
 void FWObjectDatabase::recursivelyRemoveObjFromTree(FWObject* obj,
                                                     bool remove_ref)
 {
+    obj->checkReadOnly();
+
     for (FWObject::iterator i=obj->begin(); i!=obj->end(); ++i)
         recursivelyRemoveObjFromTree( *i , true);
 
@@ -555,6 +557,8 @@ void FWObjectTreeScanner::scanAndAdd(FWObject *dst,FWObject *source)
 
 void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 {
+    string dobjId             = FWObjectDatabase::getDeletedObjectsId();
+
     if (dst==NULL)
     {
         dst=treeRoot;
@@ -567,8 +571,12 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
          */
         FWObjectDatabase *srcroot = FWObjectDatabase::cast(src->getRoot());
         FWObjectDatabase *dstroot = FWObjectDatabase::cast(dst->getRoot());
-        string dobjId             = dstroot->getDeletedObjectsId();
 
+        /* commented out 11/12/04
+         * If src has some deleted objects, we should just ignore them
+         */
+
+#if 0
         FWObject *srcdobj = srcroot->getById( dobjId );
         if (srcdobj)
         {
@@ -582,6 +590,13 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
                 }
             }
         }
+#endif
+
+        /*
+         * find deleted objects library in dst and delete objects from
+         * it if they are present and not deleted in src
+         */
+        list<FWObject*> deletedObjects;
         FWObject *dstdobj = dstroot->getById( dobjId );
         if (dstdobj)
         {
@@ -589,19 +604,30 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
             {
                 FWObject *sobj = srcMap[ (*i)->getId() ];
                 if(sobj!=NULL && sobj->getParent()->getId()!=dobjId)
-                {
-                    crp->askUser( *i, sobj );
-                    bool rof = sobj->getLibrary()->isReadOnly();
-                    sobj->getLibrary()->setReadOnly(false);
-                    srcroot->recursivelyRemoveObjFromTree(sobj);
-                    sobj->getLibrary()->setReadOnly(rof);
-                }
+                    deletedObjects.push_back(*i);
+            }
+            for (FWObject::iterator i=deletedObjects.begin(); i!=deletedObjects.end(); i++)
+            {
+                dstroot->recursivelyRemoveObjFromTree( *i );
+                dstMap[ (*i)->getId() ] = NULL;
             }
         }
     }
 
     for (FWObject::iterator i=src->begin(); i!=src->end(); i++)
     {
+        /*
+         * commented 12/04/04. We now delete "deleted objects"
+         * from libraries we are merging in before calling
+         * FWObjectDatabase::merge.  Ignoring "Deleted objects" here
+         * caused problems; in particular, deleted objects disappeared
+         * from a data file whenever it was opened. This happened
+         * because we merged user's data file into standard objects
+         * tree, so user's file was _source_ here, and deleted objects
+         * in it were ignored.
+         */
+//        if ((*i)->getId()==dobjId) continue;
+
         FWObject *dobj;
         if (FWReference::cast( *i ))
         {
