@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: dns.cpp,v 1.2 2004/09/08 05:34:52 vkurland Exp $
+  $Id: dns.cpp,v 1.4 2005/03/21 06:34:19 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -88,10 +88,8 @@ void libfwbuilder::init_dns() throw(FWException)
 
 void DNS::init()
 {
-#if !defined( HAVE_FUNC_GETHOSTBYNAME_R_6) && !defined(HAVE_FUNC_GETHOSTBYNAME_R_5)
     gethostbyname_mutex = new Mutex();
     gethostbyaddr_mutex = new Mutex();
-#endif 
 }
 
 DNS::DNS()
