@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.cpp,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: Compiler.cpp,v 1.5 2004/12/23 07:19:16 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -186,7 +186,7 @@ string Compiler::createRuleLabel(const string &txt,
 {
     ostringstream  str;
     
-    str << rule_num;
+    str << rule_num << " ";
     if (iface!=NULL) str << "(" << iface->getName() << ")";
     else             str << "(" << txt << ")";
     return str.str();
@@ -332,7 +332,7 @@ void Compiler::_expandInterface(Interface *iface,  std::list<FWObject*> &ol)
         return;
     }
 
-    physAddress *pa=iface->getPhysicalAddress();
+//    physAddress *pa=iface->getPhysicalAddress();
 /*
  * we use physAddress only if Host option "use_mac_addr_filter" of the
  * parent Host object is true
@@ -403,51 +403,15 @@ void Compiler::_expandAddressRanges(Rule *rule,FWObject *s)
 
             for (vector<IPNetwork>::iterator i=vn.begin(); i!=vn.end(); i++)
             {
-		Network *h= Network::cast(dbcopy->create(Network::TYPENAME,true) );
-                h->setAddress(i->getAddress());
+                Network *h;
+                h= Network::cast(dbcopy->create(Network::TYPENAME,true) );
+                h->setName(string("%n-")+a1.toString()+string("%") );
                 h->setNetmask(i->getNetmask());
-		h->setName(string("%")+a1.toString()+string("%") );
-		cacheObj(h); // to keep cache consistent
-		dbcopy->add(h,false);
-		cl.push_back(h);
+                h->setAddress(i->getAddress());
+                cacheObj(h); // to keep cache consistent
+                dbcopy->add(h,false);
+                cl.push_back(h);
             }
-#if 0
-/* 
- * TODO : use smarter algorithm - replace address range with
- * combination of hosts and network objects to reduce number of
- * temporary objects we create
- */
-	    IPAddress a1=AddressRange::cast(o)->getRangeStart();
-	    IPAddress a2=AddressRange::cast(o)->getRangeEnd();
-//	    ObjectGroup *hosts=ObjectGroup::cast( dbcopy->getById(dbcopy->std.HostsId,true) );
-
-	    guint32 distance=ntohl( a2.to32BitInt()-a1.to32BitInt() );
-	    if (distance>32)
-            {
-		ostringstream  str;
-		str << "Using AddressRange object in the rule "
-		    << rule->getLabel()
-		    << " is inefficient (address range will be replaced with "
-		    << distance
-		    << " individual addresses). Consider using network objects instead.";
-		warning(str.str());
-	    }
-
-	    while (a1<a2 || a1==a2)
-            {
-		IPv4 *h= IPv4::cast(dbcopy->create(IPv4::TYPENAME,true) );
-                h->setAddress(a1);
-                h->setNetmask("255.255.255.255");
-		h->setName(string("%")+a1.toString()+string("%") );
-		cacheObj(h); // to keep cache consistent
-		dbcopy->add(h,false);
-		cl.push_back(h);
-
-                if (a1==IPAddress("255.255.255.255")) break;
-
-		a1=a1+1;
-	    }
-#endif
 	}
     }
     if ( ! cl.empty() )
@@ -492,9 +456,21 @@ bool Compiler::_complexMatchWithInterface(Address   *obj1,
             IPv4 *ipv4=IPv4::cast(*k);
                     
             if ( ipv4->getAddress()==obj1_addr ) return true;
+
             IPNetwork n( ipv4->getAddress() , ipv4->getNetmask() );
-            if (recognize_broadcasts && n.getBroadcastAddress()==obj1_addr)  
-                return true;
+/*
+ * bug #1040773: need to match network address as well as
+ * broadcast. Packets sent to the network address (192.168.1.0 for net
+ * 192.168.1.0/24) go in the broadcast frame and behave just like IP
+ * broadcast packets (sent to 192.168.1.1255 for the same net)
+ *
+ */
+            if (recognize_broadcasts)
+            {
+/* obj1 is an individual address - IPv4 or Address object */
+                if (n.getAddress()==obj1_addr || n.getBroadcastAddress()==obj1_addr)
+                    return true;
+            }
         }
     }
     return false;
@@ -514,6 +490,26 @@ bool Compiler::complexMatch(Address *obj1,
                           bool recognize_broadcasts,
                           bool recognize_multicasts)
 {
+    if (Network::isA(obj1))
+    {
+        /*
+         * bug #1055937: "Any->all_multicasts not in INPUT Chain"
+         * Need to check for multicast networks. We assume they always
+         * match if obj2 is firewall
+         */
+        Network *nobj1=Network::cast(obj1);
+        if (nobj1->getAddress().isMulticast() && Firewall::isA(obj2))
+            return true;
+
+        /*
+         * need to check for network object with mask 255.255.255.255
+         * Such objects are created by the method that expands address
+         * ranges, and some often used ranges trigger that (like
+         * "255.255.255.255-255.255.255.255" or "0.0.0.0-0.0.0.0")
+         */
+        if (nobj1->getNetmask().toString()!="255.255.255.255")
+            return false;
+    }
 /*
  * if we are matching two firewalls, the positive match is only if
  * their IDs are the same (so it is the same firewall)
