@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler_ops.cpp,v 1.2 2004/09/08 05:34:53 vkurland Exp $
+  $Id: Compiler_ops.cpp,v 1.6 2005/01/25 07:33:04 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -175,13 +175,31 @@ vector<FWObject*> fwcompiler::_find_srv_intersection(Service *op1, Service *op2)
 bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
 {
     if (o1.getId()==o2.getId()) return true;
+    if (o1.isAny() && o2.isAny()) return false;
+    if ( ! o1.isAny() && o2.isAny()) return true;
+    if ( o1.isAny() && !o2.isAny()) return false;
 
     if (o1.getTypeName()==o2.getTypeName()) 
     {
 	if ( IPService::constcast(&o1)) 
         {
-/* TODO: check ip flags */
-	    return false;
+/* 
+ * Both objects are IPService
+ *
+ * can't use Service::getProtocolNumber member because it is not
+ * defined as 'const'. Can't redefine it either since that will break
+ * binary compatibility (as of v2.0.4)
+ */
+	    if (o1.getStr("fragm")!=o2.getStr("fragm") ||
+                o1.getStr("short_fragm")!=o2.getStr("short_fragm") ||
+                o1.getStr("lsrr")!=o2.getStr("lsrr") ||
+                o1.getStr("ssrr")!=o2.getStr("ssrr") ||
+                o1.getStr("rr")!=o2.getStr("rr") ||
+                o1.getStr("ts")!=o2.getStr("ts") ) return false;
+
+	    if (o1.getInt("protocol_num")==o2.getInt("protocol_num")) return true;
+	    if (o1.getInt("protocol_num")!=0 && o2.getInt("protocol_num")==0) return true;
+            return false;
 	}
 	if ( ICMPService::constcast(&o1)) 
         {
@@ -212,18 +230,27 @@ bool fwcompiler::checkForShadowing(const Service &o1,const Service &o2)
 
 	    return (srs1>=srs2 && sre1<=sre2 && drs1>=drs2 && dre1<=dre2);
 	}
+        return false;
     }
 /*
  * if objects are of the different types, then the only case allowed is
  * when one object is IPService
- *
- * TODO: need to take IP flags into account. Typical rule with
- * ip_fragment "shades" everything if we do not check flags here
- *
-    if ( ! IPService::constcast(&o1) && IPService::constcast(&o2)) return true;
-    if ( ! IPService::constcast(&o2) && IPService::constcast(&o1)) return false;
  */
-    if (!o1.isAny() && o2.isAny() ) return true;
+
+    if (IPService::constcast(&o2) && ! IPService::constcast(&o1) )
+    {
+        /* if o2 is IP Service and o1 is not, then o2 only shades o1
+         * when all flags are cleared in o2
+         */
+        return ( o2.getInt("protocol_num")==0 &&
+                 o2.getStr("fragm")!="True" &&
+                 o2.getStr("short_fragm")!="True" &&
+                 o2.getStr("lsrr")!="True" &&
+                 o2.getStr("ssrr")!="True" &&
+                 o2.getStr("rr")!="True" &&
+                 o2.getStr("ts")!="True");
+    }
+
     return false;
 
     throw FWException("trying to compare objects of incompatible types: \
@@ -246,6 +273,13 @@ bool fwcompiler::checkForShadowing(const Address &o1,const Address &o2)
         if (intf->isDyn() || intf->isUnnumbered()) return false;
     }
 
+    if (physAddress::isA(&o1) && physAddress::isA(&o2))
+    {
+        const physAddress *o1_pa =physAddress::constcast(&o1);
+        const physAddress *o2_pa =physAddress::constcast(&o2);
+        return o1_pa->getPhysAddress()==o2_pa->getPhysAddress();
+    }
+    
     IPAddress o1b;
     IPAddress o1e;
     IPAddress o2b;
@@ -378,6 +412,13 @@ bool fwcompiler::operator==(const Address &o1,const Address &o2)
             return i1->getName()==i2->getName();
         }
     }
+
+    if (physAddress::isA(&o1) && physAddress::isA(&o2))
+    {
+        const physAddress *o1_pa =physAddress::constcast(&o1);
+        const physAddress *o2_pa =physAddress::constcast(&o2);
+        return o1_pa->getPhysAddress()==o2_pa->getPhysAddress();
+    }
     
     if (AddressRange::isA(&o1))
     {
