@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp,v 1.3 2004/09/12 23:56:03 vkurland Exp $
+  $Id: PolicyCompiler.cpp,v 1.5 2004/11/11 07:00:18 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -171,14 +171,14 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
  *  important because our definition of shadowing includes the case when
  *  addresses are equal.
  */
-    if (srv1->getTypeName()==srv2->getTypeName() || (!srv1->isAny() && srv2->isAny()))
-    {
+//    if (srv1->getTypeName()==srv2->getTypeName() || (!srv1->isAny() && srv2->isAny()))
+//    {
         return ( 
             fwcompiler::checkForShadowing(*src1, *src2) && 
             fwcompiler::checkForShadowing(*dst1, *dst2) && 
             fwcompiler::checkForShadowing(*srv1, *srv2) 
         );
-    }
+//    }
 
     return false;
 }
@@ -505,10 +505,11 @@ bool PolicyCompiler::verifyCustomServices::processNext()
 }
 
 /*
- * checks if one of the children of RuleElement is a host or network object
- * with address 0.0.0.0.
+ * checks if one of the children of RuleElement is a host, IPv4 or
+ * network object with address 0.0.0.0 and netmask 0.0.0.0.
  *
  * Exceptions: 
+ *   - object 'any'
  *   - interface with dynamic address.
  */
 Address* PolicyCompiler::checkForZeroAddr::findZeroAddress(RuleElement *re)
@@ -584,12 +585,11 @@ bool PolicyCompiler::checkForZeroAddr::processNext()
         string err="Object '"+a->getName()+"'";
         if (IPv4::cast(a)!=NULL)
         {
-            err+=" (an address of";
             FWObject *p=a->getParent();
             Interface *iface=Interface::cast(p);
             if (iface!=NULL) 
             {
-                err+=" interface ";
+                err+=" (an address of interface ";
                 if (iface->getLabel()!="") err+=iface->getLabel();
                 else                       err+=iface->getName();
                 err+=" )";
@@ -858,14 +858,14 @@ string PolicyCompiler::debugPrintRule(Rule *r)
 {
     PolicyRule *rule=PolicyRule::cast(r);
 
-    FWOptions *ruleopt =rule->getOptionsObject();
+//    FWOptions *ruleopt =rule->getOptionsObject();
 
     RuleElementSrc *srcrel=rule->getSrc();
     RuleElementDst *dstrel=rule->getDst();
     RuleElementSrv *srvrel=rule->getSrv();
 
     string       iface_id = rule->getInterfaceId();
-    Interface *rule_iface = fw_interfaces[iface_id];
+//    Interface *rule_iface = fw_interfaces[iface_id];
 
     ostringstream str;
 
